package com.sitemesh.parser;

import com.sitemesh.Page;
import com.sitemesh.PageParser;
import java.io.*;
import org.xml.sax.InputSource;

/**
 * Abstract implementation of {@link com.sitemesh.PageParser}.
 * All methods convert input into <code>byte[]</code>, meaning only that
 * method need be implemented.
 *
 * @author <a href="joe@truemesh.com">Joe Walnes</a>
 * @version $Revision: 1.5 $
 *
 * @see com.sitemesh.PageParser
 */
public abstract class AbstractPageParser implements PageParser {

	/**
	 * Method to be implemented by sub-classes.
	 *
	 * @see com.sitemesh.PageParser
	 */
	public abstract Page parse( byte[] data ) throws IOException;

	public Page parse( InputStream in ) throws IOException {
		ByteArrayOutputStream data = new ByteArrayOutputStream();
		while ( true ) {
			int i = in.read();
			if ( i == -1 ) break;
			data.write( i );
		}
		return parse( data.toByteArray() ); // byte[]
	}

	public Page parse( Reader in ) throws IOException {
		CharArrayWriter data = new CharArrayWriter();
		while ( true ) {
			int i = in.read();
			if ( i == -1 ) break;
			data.write( i );
		}
		return parse( data.toCharArray() ); // char[]
	}

	public Page parse( InputSource in ) throws IOException {
		if ( in.getCharacterStream() != null ) {
			return parse( in.getCharacterStream() ); // Reader
		}
		else if ( in.getByteStream() != null ) {
			return parse( in.getByteStream() ); // InputStream
		}
		else {
			throw new IllegalStateException( "InputSource must contain Character-Stream or Byte-Stream" );
		}
	}

	public Page parse( char[] data ) throws IOException {
		ByteArrayOutputStream bytes = new ByteArrayOutputStream();
		new OutputStreamWriter( bytes ).write( data );
		return parse( bytes.toByteArray() ); // byte[]
	}

}
